#!/usr/bin/ruby -w
=begin header

launcher main program

David Maciejak
20/01/2003

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
=end

if ENV['LANG'] != nil
	ENV['LANG']='en_GB.UTF-8'
end

path_tmp=$0.split(/\//)
str=""
for i in 0..path_tmp.size-2
	str+="#{path_tmp[i]}/"
end

GNMSLIB=str

##############################################################################
require 'gtk2'
require 'logger'

##############################################################################

PIXMAP_PATH = "#{GNMSLIB}/pixmap"
CONF_DIR = "#{GNMSLIB}/conf"

ROOTMAPADDR="0.0.0.0"
ROOTMAPNAME="ROOT"

IDENT_CONF_FILE="#{CONF_DIR}/ident.conf"
GLOBAL_CONF_FILE="#{CONF_DIR}/gnms.conf"
OIDREFS_CONF_FILE="#{CONF_DIR}/snmp_oidrefs.conf"
SYSLOGREFS_CONF_FILE="#{CONF_DIR}/syslog_refs.conf"

GNMS_PROJECT_SERVER="gnms.rubyforge.org"
GNMS_PROJECT_URL=""

$pixmap_size = "32"

CRITICAL=5
MAJOR=4
MINOR=3
WARNING=2
NORMAL=1
UNMANAGED=0
UNKNOWN=-1

$status = ["CRITICAL", "MAJOR", "WARNING", "MINOR", "NORMAL", "UNMANAGED", "UNKNOWN"]
$status_color = [0xCC0000FF, 0xFF8800FF, 0xFFEE00FF, 0x69E5E5FF, 0x00CC00FF, 0xBEBEBEFF, 0x9368E5FF]
$status_value = [5, 4, 3, 2, 1, 0, -1]

$LEVEL = ["INFO","WARN","ERR","ALERT","PANIC"]
$level="WARN"
$level_view="WARN"
$talert_type=["mail", "popup", "beep", "syslog"]
$level_alert_type = Hash.new
for l in $LEVEL
  $level_alert_type[l]= [false,false,true,false] 
end

$os=Dir["#{PIXMAP_PATH}/os/*"]
$type = Dir["#{PIXMAP_PATH}/type/*"]

PROJECTNAME = "GNMS"
AUTHORS = [ "David Maciejak"]
DOCUMENTORS = []
GNMSMAJORVERSION=2
GNMSMINORVERSION=0
GNMSTAGVERSION="beta1"
if GNMSTAGVERSION != ""
	GNMSVERSION = "#{GNMSMAJORVERSION}.#{GNMSMINORVERSION}-#{GNMSTAGVERSION}"
else
	GNMSVERSION = "#{GNMSMAJORVERSION}.#{GNMSMINORVERSION}"
end

$log = Logger.new(STDOUT)
$log.level = Logger::INFO
$log.progname = PROJECTNAME

#we delete path and extension of the file
for i in [$os,$type]
  ost=[]
  for osi in i
    ostt=osi.split(/\//)
    osttt=(ostt[ostt.size - 1]).split(/\./)
    ost.push osttt[0]
  end
  if ost != nil
    if i==$os
     $os=ost.sort
    else
     $type=ost.sort
    end
  end
end

def verify_system_conf()
	ruby_cmd_version = RUBY_VERSION 
	ruby_version="^1\.8"
	if ruby_cmd_version.match(ruby_version)
		if !Gtk.check_version(2,4,0)
			return 0
		else
			$log.error("This tool need at least GTK version 2.4")
		end		
	end
	return 1
end

def txtlvl_to_loggerlvl(str)
	case str
	when "Debug"
		return Logger::DEBUG
	when "Info"
		return Logger::INFO
	when "Warning"
		return Logger::WARN
	when "Error"
		return Logger::ERROR
	else
		return Logger::INFO
	end
end

####################################################################################
#
#	go!
#

if ENV["USER"] != "root"
	$log.error("You are not logged as root, you can't used full features of gnms ...")
	exit
end

#verify ruby/gtk/gnome conf
unless verify_system_conf == 0
	$log.error("System configuration is not ok")
	exit
else
	$log.info("System configuration is ok")

##############################################################################
#here we read plugins too
require "#{GNMSLIB}"+'/module/main'
##############################################################################


Gtk::RC.parse_string <<EOS

style "tooltips"

{
  bg[NORMAL] = {0.94, 0.90, 0.54}
}

widget "gtk-tooltips" style "tooltips"
EOS

  Gtk.init

  window = Gtk::Window.new
  window.set_default_size(300,150)
  window.set_window_position(Gtk::Window::POS_CENTER)
  window.set_title "GNMS"
  window.set_decorated(false)
  window.set_modal(true)
  window.show

  hbox = Gtk::HBox.new
  hbox.show
  hbox.add(Gtk::Image.new("#{PIXMAP_PATH}/logo.jpg"))
  window.add hbox
  window.show_all

  launch_gnms_main(window)

  Gtk.main
end
