require "#{GNMSLIB}"+'module/gui/node_view'
require "#{GNMSLIB}"+'module/gui/node_network_view'
require "#{GNMSLIB}"+'module/gui/node_host_view'

class CanvasMap < Gtk::VBox

	def fullscreen?()
		return @fullscreen==true
	end

	def toggle_fullscreen()
      if @fullscreen == false
		   @fullscreen=true	
			$win.hide_menu()
	      $win.fullscreen()
      else
	      @fullscreen=false
			$win.show_menu()
	      $win.unfullscreen()
      end
	end

#
#	set the background image
#
def set_background_image(image_file)
	if !defined?(@bg_img) || (@bg_img == nil)
   	  @bg_img=Gnome::CanvasGroup.new(@canvas.root,{:x => 0.0,:y => 0.0})
	end
	if (image_file != nil) && FileTest.exist?(image_file)
		im=Gdk::Pixbuf.new(image_file)
		@bg_img.lower_to_bottom() 
		if defined?(@img) && (@img != nil)
			@img.hide
			@img.destroy
		end
		@img=Gnome::CanvasPixbuf.new(@bg_img,
                                    		  :pixbuf => im,
				      								  :x => 0,
                                    		  :y => 0,
														  :width_set => true,
														  :height_set => true,
                                    		  :width => @canvas.width,
                                    		  :height => @canvas.height)
		@img.lower_to_bottom()
		@bg_img.show
		@bg_image_enable=true
	end
end

def background_image_isset?()
	#return @bg_img != nil
	return @bg_image_enable
end

#
#	unset the background image
#
def unset_background_image()
	if defined?(@bg_img)
		@bg_img.hide() unless @bg_img==nil
		@bg_image_enable=false
	end
end

  def get_selected_node()
	 nodeselected=[]
	 if $map != nil
	    $network[get_map()].host.each_value {|n|
	   	 if n.node_view.selected?()
				nodeselected.push(n)
			 end
	    }
	 else
		if $network[ROOTMAPADDR].node_view.selected?()
			nodeselected.push($network[ROOTMAPADDR])
		end
	 end
	 return nodeselected
  end

	def node_view_zoom_in()
		if $map == nil
			$config.pixmap_size+=5
			$network[ROOTMAPADDR].node_view.resize()
		else
			if $network[get_map()].node_size == nil
				$network[get_map()].node_size=$config.pixmap_size+5
			else
				$network[get_map()].node_size+=5
			end
	   	  $network[get_map()].host.each_value {|n| 
			  n.node_view.resize()
			}
		end
	end

	def node_view_zoom_out()
		if $map == nil
			$config.pixmap_size-=5 unless $config.pixmap_size < 10
			$network[ROOTMAPADDR].node_view.resize()
		else
			if $network[get_map()].node_size == nil
				$network[get_map()].node_size=$config.pixmap_size-5 unless $config.pixmap_size < 10
			else
				$network[get_map()].node_size-=5 unless $network[get_map()].node_size < 10
			end
	        $network[get_map()].host.each_value {|n| 
			  n.node_view.resize()
			}
		end
	end

	def nb_selected()
		return get_selected_node().size()
	end

  def key_press(canvas, event)
	estring=Gdk::Keyval.to_name(event.keyval)
        print "code touche1: ",event.keyval," = ",estring,"\n"

    x, y = canvas.scroll_offsets
    case event.keyval
	 when Gdk::Keyval::GDK_KP_Add
		node_view_zoom_in()
	 when Gdk::Keyval::GDK_KP_Subtract
		node_view_zoom_out()
	 when Gdk::Keyval::GDK_Delete
		del_selected_node()
	 when Gdk::Keyval::GDK_d
		del_selected_node()
	 when Gdk::Keyval::GDK_i
		#up
      get_selected_node().each {|n| 
		  n.node_view.move_up()
		}
	 when Gdk::Keyval::GDK_k
		#down
      get_selected_node().each {|n| 
		  n.node_view.move_down()
		}
	 when Gdk::Keyval::GDK_j
		#left
      get_selected_node().each {|n| 
		  n.node_view.move_left()
		}
	 when Gdk::Keyval::GDK_l
		#right
      get_selected_node().each {|n| 
		  n.node_view.move_right()
		}
    when Gdk::Keyval::GDK_Up
      canvas.scroll_to(x, y - 20)
    when Gdk::Keyval::GDK_Down
      canvas.scroll_to(x, y + 20)
    when Gdk::Keyval::GDK_Left
      canvas.scroll_to(x - 10, y)
    when Gdk::Keyval::GDK_Right
      canvas.scroll_to(x + 10, y)
    when Gdk::Keyval::GDK_Page_Up
		set_map($network[$map].map) unless $map == nil
    when Gdk::Keyval::GDK_f
		toggle_fullscreen()
    end   	
  end

  def get_root()
	@canvas.root
  end

  def redraw_all()
	@canvas.update_now()
  end

  def set_rgb(r, g, b)
	 style=Gtk::Style.new
	 style.set_bg(Gtk::STATE_NORMAL, r, g, b)
	 style.set_bg(Gtk::STATE_ACTIVE, r, g, b)
	 style.set_bg(Gtk::STATE_PRELIGHT, r, g, b)
	 @canvas.set_style(style)
  end

	def set_size(w, h)
		#@canvas.set_size_request(w, h)
    	@canvas.set_scroll_region(0, 0, w, h)
		if @bg_image_enable && (@img != nil)
			@img.width=@canvas.width
			@img.height=@canvas.height
		end
	end

  def initialize(aa)
    super()
	 @bg_image_enable=false
    @dragging = false
	 @cursor_x=nil
	 @cursor_y=nil
	 @drawing_selection=false
	 @selection=nil
    @fullscreen=false
	 #need var to bypass gtk problem: item_at does not work on canvasWidget?

    show()

    # Create the canvas
    @canvas = Gnome::Canvas.new(aa)
	 #here we set the background color
	 set_rgb($config.rgb[0].to_f, $config.rgb[1].to_f, $config.rgb[2].to_f)

    # Setup canvas items
    root = @canvas.root
    # Layout the stuff

    table = Gtk::Table.new(2, 2, false)
    ##table.set_row_spacings(4)
    ##table.set_column_spacings(4)
    pack_start(table,true)
    #pack_start(@canvas, true, true)

    table.show()
    table.attach(@canvas,
                 0, 1, 0, 1,
                 Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK,
                 Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK,
                 0, 0)

    @canvas.show()

    @canvas.signal_connect_after("key_press_event") do |item, event|
      key_press(item, event)
    end

    @canvas.signal_connect("button_press_event") do |w, event|
		xw, yw = w.window_to_world(event.x,event.y)
 		cvitem=w.get_item_at(xw, yw)

		if (cvitem == nil) || (cvitem.parent == @bg_img if defined?(@bg_img))
			if $map == nil
				$network[ROOTMAPADDR].node_view.unselect()
			else
	      	$network[get_map()].host.each_value {|node|
	   			node.node_view.unselect()
	      	}
			end
	     	if (event.event_type == Gdk::Event::BUTTON_PRESS)
		      if event.button == 3 #bouton droite
		 			#show popup menu
               b = event.button
               t = event.time

               popup_menu = Gtk::Menu.new
               popup_menu.append(add_node=Gtk::MenuItem.new("Add host"))
               popup_menu.append(find_node=Gtk::MenuItem.new("Find node"))
		  			if $event_win.visible? 
	                  popup_menu.append(ewin_hide=Gtk::MenuItem.new("Hide event log"))
		  			else
	                  popup_menu.append(ewin_show=Gtk::MenuItem.new("Show event log"))
		  			end
		  			popup_menu.append(layout_grid=Gtk::MenuItem.new("Grid layout"))
		  			popup_menu.append(map_up=Gtk::MenuItem.new("Go up"))
					if $canvas.fullscreen?()
						popup_menu.append(Gtk::SeparatorMenuItem.new)
			  			popup_menu.append(exit_fullscreen=Gtk::MenuItem.new("Unfullscreen"))
		  				exit_fullscreen.signal_connect('button-press-event') {
							toggle_fullscreen()
	               	popup_menu.destroy
               	}
					end

		  			if $map == nil
						layout_grid.set_sensitive(false)
						map_up.set_sensitive(false)
						add_node.set_sensitive(false)
		  			end
               popup_menu.show_all
               popup_menu.popup(nil,nil,b,t)

		  			add_node.signal_connect('button-press-event') {
		          	addnodewin
	               popup_menu.destroy
               }
               find_node.signal_connect('button-press-event') {
			  			findnodewin
	               popup_menu.destroy
               }
		  			if (ewin_hide)
		     			ewin_hide.signal_connect('button-press-event') {
			  				$event_win.hide()
                     popup_menu.destroy
					}
		  			end
					if (ewin_show)
		   			ewin_show.signal_connect('button-press-event') {
			  			 $event_win.show()
            		 popup_menu.destroy
         			}
					end
 					layout_grid.signal_connect('button-press-event') {
		      		  layout_grid($map)
            		  popup_menu.destroy
					}
					map_up.signal_connect('button-press-event') {
						if $map != nil
		      		  set_map($network[$map].map)
						end
           		   popup_menu.destroy
                }
	       else
				#draw selection with other button
				@drawing_selection=true
				@cursor_x=event.x
				@cursor_y=event.y
	       end
	   end
	end
    end

    @canvas.signal_connect("button_release_event") do |w, event|
		if @drawing_selection
			#we select node in rectangle selection
			cursor_x, cursor_y = w.window_to_world(@cursor_x,@cursor_y)
			x2, y2 = w.window_to_world(event.x,event.y)

			if cursor_x>x2
			 xpos=x2.to_i
			 x2=cursor_x
			else
			 xpos=cursor_x.to_i
			end
			while xpos < x2.to_i
	   	  if cursor_y>y2
				ypos=y2.to_i
				y2=cursor_y
	   	  else
	         	  ypos=cursor_y.to_i
	   	  end
    	   	  while ypos < y2.to_i
					 if $map == nil
						 node=$network[ROOTMAPADDR]
						 if xpos<node.x && x2>node.x+$config.pixmap_size && ypos<node.y && y2>node.y+$config.pixmap_size+10
		      			 #pos point is in node zone
		      			 node.node_view.select()
		   			 end
					 else
	      			 $network[get_map()].host.each_value {|node|
		   			 if xpos<node.x && x2>node.x+$config.pixmap_size && ypos<node.y && y2>node.y+$config.pixmap_size+10
		      			 #pos point is in node zone
		      			 node.node_view.select()
		   			 end
	      			 }
					 end
	      		 ypos+=$config.pixmap_size+10
	   	  		end
	   		 xpos+=$config.pixmap_size
				end
	       @drawing_selection=false
		end
		 #hide rectangle selection
		 @selection.hide() unless @selection == nil
		 #redraw links cause nodes have moved
		 node_tmp=nil
		 get_selected_node().each {|n|
			node_tmp=n
			if n.nb_links > 0
				n.get_links().each {|lnk|
					lnk.remove_link
					lnk.display()
				}
			end
		 }
    end

    @canvas.signal_connect("motion_notify_event") do |w, event|
       if @drawing_selection	
			@selection.hide() unless @selection == nil
			cursor_x, cursor_y = w.window_to_world(@cursor_x,@cursor_y)
			x2, y2 = w.window_to_world(event.x,event.y)
			@selection=Gnome::CanvasRect.new(root,
                                		  {:x1 => cursor_x,
                                          	  :y1 => cursor_y,
                                          	  :x2 => x2,
                                          	  :y2 => y2,
					   									  :outline_color => "red",
                                          	  :width_units => 1.0})
       end
    end

#@canvas.signal_connect("expose_event") do |w,e|
#	print w," ",e,"\n"
#end

#    w = Gtk::HScrollbar.new(@canvas.hadjustment)
#    table.attach(w,
#                 0, 1, 1, 2,
#                 Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK,
#                 Gtk::FILL,
#                 0, 0);
#    w.show()

#    w = Gtk::VScrollbar.new(@canvas.vadjustment)
#    table.attach(w,
#                 1, 2, 0, 1,
#                 Gtk::FILL,
#                 Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK,
#                 0, 0);
#    w.show()

    @canvas.flags = Gtk::Widget::CAN_FOCUS
    @canvas.grab_focus()
  end
end
