require 'gtk2'
require 'gnome2'
require "#{GNMSLIB}"+'module/node_property'
require "#{GNMSLIB}"+'module/node'
require "#{GNMSLIB}"+'module/netlocal'
require "#{GNMSLIB}"+'module/nodelistener'
require "#{GNMSLIB}"+'module/gui/adddelnode'
require "#{GNMSLIB}"+'module/gui/base_window'
require "#{GNMSLIB}"+'module/gui/alarm'
require "#{GNMSLIB}"+'module/gui/clist.rb'

class EventWindow

EVENT_WINDOW_TITLE="Event Browser"

#
#	return if window is visible
#
def visible?()
	return @window.visible?()
end

#
#	show the window
#
def show()
	if $canvas.fullscreen?()
	  @window.set_modal(true)
	  @window.set_transient_for($win)
	end
	return @window.show()
end

#
#	hide the window
#
def hide()
	if $canvas.fullscreen?()
	  @window.set_modal(false)
	  @window.set_transient_for(nil)
	end
	return @window.hide()
end

def filter_by_sev(sev)
	$level_view = sev
	if @filter_node
		filter_by_node(@filter_node)
	else
		filter_by_node()
	end
end

#
#	show logs for this specific node, node=nil to show all logs
#
def filter_by_node(node=nil)
	@list.clear()
	if node
	  @filter_active=true
	  @filter_node=node
	  @event_strings.each {|pos, category, src_module, time, ip, summary|
		 if filter_on_node(ip) && ($LEVEL.index(category)>=$LEVEL.index($level_view))
			text = []
   		text.push pos
   		text.push category
   		text.push src_module
   		text.push time
   		text.push ip
   		text.push summary
			@list.prepend(text)
		 end
	  }
	  @window.set_title(EVENT_WINDOW_TITLE + " filter #{$level_view} from #{node.ip}")
	else
	  @filter_active=false
	  @filter_node=nil
	  @event_strings.each {|pos, category, src_module, time, ip, summary|
puts pos, category, time, ip, summary
		if ($LEVEL.index(category)>=$LEVEL.index($level_view))
			text = []
   		text.push pos
   		text.push category
   		text.push src_module
   		text.push time
   		text.push ip
   		text.push summary
			@list.prepend(text)
		end
	  }
	  @window.set_title(EVENT_WINDOW_TITLE)
	end
end

#
#	define if ip_str is part of the filter tag
#
def filter_on_node(ip_str)
	if @filter_node == nil
		return false
	end
	if exist_network(@filter_node.ip)
		#we want to retrieve events from all a network
		return $network[@filter_node.ip].contain_host(ip_str)		
	end
	return ip_str == @filter_node.ip
end

#
# add event in list
#
def add_event(src_module, category, ip_str, summary, datetime=nil)
  text = []
  index_pos=@event_strings.length()+1
  text.push index_pos.to_s
  text.push category
  text.push src_module
	if datetime==nil
	  time_tmp=Time.new.to_s
	  text.push time_tmp
	else
	 text.push datetime
	end
  text.push ip_str
  text.push summary
  @event_strings.push(text)
  if filter_on_node(ip_str) || @filter_active == false
	 if $LEVEL.index(category)>=$LEVEL.index($level_view)	
	  @list.prepend(text)
	 end
  end

  if $LEVEL.index(category)>=$LEVEL.index($level)
    i=0
    $level_alert_type[category].each do |alert|
      if alert == true
        case i
          when 0 then #MAIL  
            alarm_mail("#{text[2]} #{text[3]} #{text[4]} #{text[5]}")
          when 1 then #POPNORMAL
				puts "here dans popup"
            alarm_popup("#{text[2]} #{text[3]} #{text[4]} #{text[5]}")
          when 2 then #BEEP
            alarm_beep()
          when 3 then #SYSLOG
	    		text=sprintf("module='%s', severity='%s', time='%s', src='%s', descr='%s'", src_module, category, time_tmp, ip_str, summary)
	    		alarm_syslog(text)
        end
      end
      i+=1
    end
  end
  update_statusbar()
end

def delevent()
	 iter = @list.selection.selected
	 @list.remove(iter) unless iter == nil
end

def initialize()

  @statusbar=nil
  @list=nil
  @event_strings=[]
  @filter_active=false
  @filter_node=nil

  @window=Gtk::Window.new
  @window.signal_connect("delete_event") do
		self.hide()
  end

  @window.signal_connect("key_press_event") {|w,e|
    if e.keyval == Gdk::Keyval::GDK_Escape
		self.hide()
    end
  }

  @window.set_size_request(600, 100)
  @window.set_title(EVENT_WINDOW_TITLE)
  img = Gtk::Image.new "pixmap/logo_icon.jpg"
  @window.set_icon(img.pixmap)

  box1 = Gtk::VBox.new(false, 0)
  @window.add(box1)

  box1.show
  box11 = Gtk::VBox.new(false, 0)
  box11.show

#------------begin scrolledbar--------------------------------------
  scrolled_window = Gtk::ScrolledWindow.new(nil, nil)
  scrolled_window.border_width=0
  scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC,
                             Gtk::POLICY_AUTOMATIC)
  # scrolled_window.vscrollbar.unset_flag(Gtk::CAN_FOCUS)
  box1.pack_start(scrolled_window, true, true, 0)
  
  scrolled_window.add_with_viewport(box11)
  box11.set_focus_vadjustment(scrolled_window.vadjustment)
  scrolled_window.show
#----------------end scrolledbar-----------------------------------


@list = Gtk::CList::new(["Event ID", "Level", "Module","Timestamp", "Node", "Summary" ])
@list.set_sort_column(1,4)
#@list.selection.set_mode(Gtk::SELECTION_MULTIPLE)
# Create a popup menu
menu = Gtk::Menu.new

item_clear = Gtk::MenuItem.new("Clear event(s)")
item_clear.signal_connect("activate") { @list.clear() }
menu.append(item_clear)

item_remove_filter = Gtk::MenuItem.new("Remove src filter")
item_remove_filter.signal_connect("activate") { filter_by_node() }
menu.append(item_remove_filter)


map_menu_severity=Gtk::MenuItem.new("Severity threshold")
map_menu_severity_list=Gtk::Menu.new
radio_menu_grp=nil
for level in $LEVEL
	rmi=Gtk::RadioMenuItem.new(radio_menu_grp,"#{level}")
	if level == $level_view
		rmi.set_active(true)
	end
   rmi.signal_connect("activate") do |widget|
			if widget.active?
		       filter_by_sev(widget.child.label)
			end
    end
	radio_menu_grp=rmi.group
	map_menu_severity_list.append(rmi)
end
map_menu_severity.set_submenu(map_menu_severity_list)
menu.append(map_menu_severity)

menu.show_all

@list.signal_connect("button_press_event") do |widget, event|
  if event.kind_of? Gdk::EventButton and event.button == 3
		if @list.nb_event > 0
			item_clear.set_sensitive(true)
		else
			item_clear.set_sensitive(false)
		end
		if @filter_active
			item_remove_filter.set_sensitive(true)
		else
			item_remove_filter.set_sensitive(false)
		end
    menu.popup(nil, nil, event.button, event.time)
  end
end

@list.signal_connect("key_press_event") {|w, e| 
  if (e.keyval == Gdk::Keyval::GDK_Delete) || (e.keyval == Gdk::Keyval::GDK_d)
    delevent()
  end
}

 @list.show
 box11.add @list
 
 add_event("core","INFO",iplocal()[0].split("/")[0], "GNMS launch")

#---------------------begin statusbar---------

    @statusbar = Gtk::Statusbar.new
    
    vvbox = Gtk::VButtonBox.new
    vvbox.border_width=0
    vvbox.set_layout_style(Gtk::ButtonBox::END)
    vvbox.set_spacing(0)
    vvbox.set_size_request(200, 20)
    vvbox.show

    box1.pack_start(@statusbar,false,false,0)
    
    @statusbar.show
    @statusbar.signal_connect("text_popped") do |o, mid, text|
      popped(mid,text)
    end
    @statusbar.signal_connect("text_pushed") do |o, mid, text|
      pushed(mid,text)
    end

#---------------------end status bar----------

end

#
#update status bar (active host) / (total host)
#
def update_statusbar
	hup=0
	hdown=0
	if @statusbar != nil
	   $host.each_key {|ip|
		   if $host[ip].status == NORMAL
			   hup=hup+1
		   else
			   hdown=hdown+1
		   end
	   }
           @statusbar.push(1, "Active Host(s) "+"#{hup}/#{$host.size}")
	end
end

 def save_event()
	iter_zero=@list.model.get_iter("0")
	if iter_zero != nil
	  bbeg= @list.get_value(iter_zero,3)
	  vall=@list.nb_event-1
	  bend=@list.get_value(@list.model.get_iter("#{vall}"),3)
	  bbeg=bbeg[4,bbeg.size]
	  bend=bend[4,bend.size]

	  fic = File.new("log/#{bend}-#{bbeg}",'w')
	  for row in 0..@list.nb_event-1
	#    print @list.get_value(@list.model.get_iter("#{row}"),3),"\t"
	#    print @list.get_value(@list.model.get_iter("#{row}"),2),"\t"
	#    print @list.get_value(@list.model.get_iter("#{row}"),4),"\n"
   	 fic.puts(@list.get_value(@list.model.get_iter("#{row}"),3)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),2)+"\t"+@list.get_value(@list.model.get_iter("#{row}"),4))
	  end
	  fic.close
	end
end

def pushed(mid,text)
end

def popped(mid,text)
end

end
