require 'gtk2'
require 'gdk_pixbuf2'
require 'libart2'
require 'gnomecanvas2'
require 'syslog'
require 'tempfile'

require "#{GNMSLIB}"+'module/oid_ref'
require "#{GNMSLIB}"+'module/oid_ref_manage'

require "#{GNMSLIB}"+'module/syslog_ref'
require "#{GNMSLIB}"+'module/syslog_ref_manage'

require "#{GNMSLIB}"+'module/node'
require "#{GNMSLIB}"+'module/node_host'
require "#{GNMSLIB}"+'module/node_network'
require "#{GNMSLIB}"+'module/link'
require "#{GNMSLIB}"+'module/gui/event_window'
require "#{GNMSLIB}"+'module/gui/canvas_map'
require "#{GNMSLIB}"+'module/server/syslog/syslog_capture'

################################################################################
# Main Event Window
################################################################################


def get_color_from_status(status)
	case status
	when "CRITICAL"
		return $red
	when "MAJOR"
		return $orange
	when "WARNING"
		return $yellow
	when "MINOR"
		return $blue
	when "NORMAL"
		return $green
	when "UNMANAGED"
		return $grey
	when "UNKNOWN"
		return $purple
	end
end

#
#	convert status to level
#  can return a level or '' if not convert or "UNKNOWN", "UNMANAGED" if "UNMANAGED"
def get_level_from_status(status)
  case status
  when "UNMANAGED"
  	return "UNMANAGED"
  when "CRITICAL"
  	return "PANIC"
  when "MAJOR"
  	return "ALERT"
  when "WARNING"
  	return "ERR"
  when "MINOR"
  	return "WARN"
  when "NORMAL"
  	return "INFO"
  else
  	return ""
  end
end

#
#	str -> boolean
#
def to_b(str)
  return str == "true"  
end

def add_monitoring_thread(str, thr)
	$log.debug("Starting thread #{str}")
	$thread_list[str]=thr
	$threads.add(thr)
end

#
#	remove thread from thread group
#
def del_monitoring_thread(str)
		ThreadGroup.new.add($thread_list[str])
		$thread_list.delete($thread_list[str])	
end
#
#	popup to confirm application exit
#
def confirm_exit_window()
	if to_b($config.confirm_quit)
   	dialog = Gtk::MessageDialog.new($win, Gtk::Dialog::DESTROY_WITH_PARENT,
                           	  Gtk::MessageDialog::QUESTION,
                           	  Gtk::MessageDialog::BUTTONS_YES_NO,
                           	  "Do you really want to quit ?")
		dialog.run do |r|
			dialog.destroy
			if r == Gtk::Dialog::RESPONSE_NO then
				return true
			else
				return false
			end	
		end
	else
		return false
	end
end

#
#	fill ip str with prepending zero chars
#
def fill_ip_str(ip)
	table=ip.split(/\./)
	if table.length != 4
		return nil
	end
	ip_tmp=""
	for i in 0..table.size-1
		ip_tmp+="0"*(3-table[i].length)+table[i]
		ip_tmp+="." unless i==table.size-1
	end
	return ip_tmp
end

#
#	return true if arg is an ip addr
#	need: an arg
#
def isValidIp(ip)
  lip = ip.split(/\./)
  if lip.size != 4
     return false
  end
  lip.each do |val|
	if !isInteger(val) || (val.to_i > 255 || val.to_i < 0)
		return false
	end
  end
  return true
end

#
#	return true if arg is an integer
#	need: an arg
#
def isInteger(str)
	return str.match(/^\d+$/)
end

def get_map_width()
	if $win.window != nil
		return $win.window.geometry[2]
	else
		return $root_width
	end
end

def get_map_height()
	if $win.window != nil
		return $win.window.geometry[3]
	else
		return $root_height
	end
end

def get_default_map_width()
	if $config.default_width == nil || $config.default_width == ""
		return Gdk::screen_width/2
	else
		return $config.default_width.to_i
	end
end

def get_default_map_height()
	if $config.default_height == nil || $config.default_height == ""
		return Gdk::screen_height/2
	else
		return $config.default_height.to_i
	end
end

def del_selected_node()
	mapp=get_map()
	if mapp != ROOTMAPADDR
		nodes=[]
	   $network[mapp].host.each_value {|node|
	   	if node.node_view.selected?()
				nodes.push(node)
			end
	   }
		delnodewin(nodes) unless nodes.size() == 0
	end
end

#
#	try to display node of map arg as a grid
#	need: an ip addr map
#
def layout_grid(map)
	if map != nil

	 root_width=$win.window.geometry[2]
	 root_height=$win.window.geometry[3]
	 x=0
	 y=0
	
	 $network[map].host.each_value {|node|
		 node_size=node.node_view.get_node_size()
		 node.node_view.move(x-node.x, y-node.y)
		 x+=node_size+root_width/node_size
		 if x >= root_width
			x=0
			y+=node_size+node_size/2+root_height/node_size	
		 end
		 if node.nb_links > 0
				node.get_links().each {|lnk|
					lnk.remove_link
					lnk.display()
				}
		  end
	 }
	end
end

#
#	return the number of host
#
def n_host()
	return $host.size
end

#
#	return the number of network
#
def n_network()
	return $network.size
end

#
#	return true if host ip addr is known
#	need: ip
#
def exist_host(ip)
	return $host[ip] != nil
end

#
# del host from a network
#
def del_host(ip)
	#we first update network list
	$network[$host[ip].map].del_host(ip)
	$host.delete ip
end

#
# add host to a network
#
def add_host(ip, mask, map=ROOTMAPADDR)
	if $host[ip] == nil
	   $host[ip]=Host.new(ip, mask, map)
	   #we update network host list
	   $network[map].add_host($host[ip])
		$host[ip].node_view.show() if map==get_map()
	   return true
	end
	return false
end

#
#	return true if network ip addr is known
#	need: ip
#
def exist_network(ip)
	return $network[ip] != nil
end

#
#	delete a network
#	need: ip
#
def del_network(ip)
	if ip != ROOTMAPADDR
	  $network[$network[ip].map].del_host(ip) unless $network[$network[ip].map] == nil
	  $network[ip].host.each_key {|id|
		  $host[id].delete() unless $host[id] == nil
	  }
	  $network.delete ip
	end
end

#
#	add a network
#	need: ip, mask, map
#
def add_network(ip, mask, map)
	if map != nil && $network[ip] == nil
	   $network[ip]=Network.new(ip, mask, map)
		#add network in network map
		$network[map].add_host($network[ip])
	   return true
	end
	return false
end

#
#	return table of network ip
#
def get_network()
	hnet=[]
	$network.each_value {|node|
		#if node.name?
		#	hnet.push node.name
		#else
			hnet.push node.ip
		#end
	}
	return hnet
end

#
#	set current network map window title
#
def set_map_window_title(txt)
	$win.set_title(txt)
end

#
#	return current network map
#
def get_map()
	if $map == nil
		return ROOTMAPADDR
	else
		return $map
	end
end

#
# set current map by is IP addr
#
def set_map(mp=nil)
	$canvas.get_selected_node().each {|n|
		n.node_view.unselect()
	} unless $canvas == nil
	#first set map
	if $map == nil && mp ==nil
			#puts "1"
			$network[ROOTMAPADDR].node_view.show()
			$network[ROOTMAPADDR].map_view.hide()
	else
		if $map == nil
			#puts "2"
			#only the network root can have a nil map
			$network[ROOTMAPADDR].host.each_value {|node|
				node.node_view.hide_text()
			}
			$network[ROOTMAPADDR].node_view.hide()
		else
			#puts "3"
			$network[$map].host.each_value {|node|
				node.node_view.hide_text()
			}
			$network[$map].map_view.hide()
		end
		if mp == nil
			#puts "4"
			#only the network root can have a nil map
			$network[ROOTMAPADDR].node_view.resize()
			$network[ROOTMAPADDR].reload()
			if $network[ROOTMAPADDR].set_label_visibility()
				$network[ROOTMAPADDR].change_label($config.network_label_view)
				$network[ROOTMAPADDR].node_view.show_text()
			else
				$network[ROOTMAPADDR].node_view.hide_text()
			end
			$network[ROOTMAPADDR].node_view.show()
			if $config.bg_type == "color"
				$canvas.unset_background_image()
				set_background_color($config.rgb)
			else
				$canvas.set_background_image($config.image_path)
			end
			$win.hide_menu_entry()
		else
			#puts "5"
			#for each node of the new map, we refresh size icon, icon type and label type
			$network[mp].host.each_value {|n|
			   	n.node_view.resize()
					n.reload()
					if n.set_label_visibility()
						if n.class == Network
					   	n.change_label($config.network_label_view)
						elsif n.class == Host
							n.change_label($config.host_label_view)
							n.refresh_dns_name()
						end
						n.node_view.show_text()
					else
						n.node_view.hide_text()
					end
					#show the node icon view
					n.node_view.show()
			}
			$network[mp].map_view.show()
#puts $network[mp].map_bg_type

#test $map et mp bg_type ?
			if $network[mp].map_bg_type != nil
				if $network[mp].map_bg_type == "color"
					if ($network[mp].map_bg_data != $config.rgb) || ($config.bg_type == "image")
						puts "avant unset background image"
						$canvas.unset_background_image()
						set_background_color($network[mp].map_bg_data)
					end
				else
					if ($network[mp].map_bg_data != $config.image_path)
#puts "here1"
						$canvas.set_background_image($network[mp].map_bg_data)
					end
				end
			else
				if $config.bg_type == "color"
					$canvas.unset_background_image()
					set_background_color($config.rgb)
				else
					if (!$canvas.background_image_isset?()) || ($network[mp].map_bg_data != $config.image_path)
						puts "pas de bg image"
						$canvas.set_background_image($config.image_path)
					end
				end
			end
			$win.show_menu_entry()
		end
		$map=mp
		set_map_window_title("Map #{mp}")
	end
end

#
#	set map window background rgb color
#
def window_set_rgb(r, g, b)
  style=Gtk::Style.new
  style.set_bg(Gtk::STATE_NORMAL, r, g, b)
  style.set_bg(Gtk::STATE_ACTIVE, r, g, b)
  style.set_bg(Gtk::STATE_PRELIGHT, r, g, b)
  $win.set_style style
end

#
#	set map window background color
#
def set_background_color(rgb)
	r,g,b=rgb.to_a()
	r=r.to_f
	g=g.to_f
	b=b.to_f
	#print r," ",g," ",b,"\n"
	window_set_rgb(r, g, b) unless $win == nil
	$canvas.set_rgb(r, g, b) unless $canvas == nil
end

#
#	select all node of the map
#
def select_all_node()
  $network[get_map()].host.each_value {|n|
 	  n.node_view.select()
  }
end

#
#	invert node selection of the map
#
def invert_selected_node()
  $network[get_map()].host.each_value {|n|
	  if n.node_view.selected?()
 		  n.node_view.unselect()
	  else
 		  n.node_view.select()
	  end
  }
end

#
# create main network map window
#
def map_window()
  nbuttons = n_host()
  $win = App.new("Gnms", "Map")

  $win.set_default_size(get_default_map_width(), get_default_map_height())
  #here we set the background color
  window_set_rgb($config.rgb[0].to_f, $config.rgb[1].to_f, $config.rgb[2].to_f)

  $win.signal_connect("delete_event") {
   	$root_width=$win.window.geometry[2]
	   $root_height=$win.window.geometry[3]
		confirm_exit_window()
	}
  $win.signal_connect("destroy") {
		$win.quit_all()
  }

#fixme when modifying main window
  $win.signal_connect("configure_event") {|w,e|
	 #if $win != nil
		#print "dans map_window ", $root_width," ",$root_height," ",e.width," ",e.height,"\n"
		#$win.proportional_node_position($root_width,$root_height,e.width,e.height)
		#$root_width=e.width
		#$root_height=e.height
	 #end
  }

  #FIXME2
  #  $win.set_policy (true,false,true)

  if ARGV[0] != nil
    if FileTest.exist?(ARGV[0])
      read_xml(ARGV[0])
    else
      print "Error: #{ARGV[0]} is not an host db file","\n"
      read_xml()
    end
  else
    read_xml()
  end 

  if n_host() == 0
    for ipnet in iplocal
      #syntax iplocal ip/netmask
      ipnet_t=ipnet.split(/\//)
      ip=ipnet_t[0]
      netmask=ipnet_t[1]
      if !exist_host(ip)
			add_host(ip, netmask)
      end
    end
  end
  
  #set current map
  map_tmp=nil
  if $config.default_map != ""
	 if exist_network($config.default_map)
		 map_tmp=$config.default_map
	 end
  end
  set_map(map_tmp)

  $win.set_icon_name("MAP")
  img = Gtk::Image.new "pixmap/logo_icon.jpg"
  $win.set_icon(img.pixmap)
  $win.show
  $root_width=$win.window.geometry[2]
  $root_height=$win.window.geometry[3]

  #here we set the pixmap background
  if $config.image_path != "" && $config.bg_type == "image"
  	if FileTest.exist?($config.image_path)
  		$canvas.set_background_image($config.image_path)
	else
		$log.warn("Warning: Image background #{$image_path} doesnt exist !")
	end
  end

end

#
#	main
#
def launch_gnms_main(window)

  Gnome::Program.new(PROJECTNAME, "Map")

  $config=ConfigGlobal.new()
  #read config

  $config.read_config()
  if ! FileTest.exist?(GLOBAL_CONF_FILE)
    $config.show()
  end

  #set saved log level
  $log.level = txtlvl_to_loggerlvl($config.log_level)

  #verify nmap version
  $config.nmap_vers=nmap_version()
	if $config.nmap_vers == nil
			$log.warn("You should install NMAP version from http://www.insecure.org/nmap/ to access full features")
	else
  		if  $config.nmap_vers.to_i() < 4
			$log.info("You should update your NMAP version to 4.x from http://www.insecure.org/nmap/")
		end
	end

  #create host hash
  $host=Hash.new

  #create network hash
  $network=Hash.new

  #create link hash for global operation
  $links=[]

  #load oid ref for snmp monitoring
  $oid_refs=Hash.new
  read_oidref_conf_file()

  #load syslog ref for syslog monitoring
  $syslog_refs=Hash.new
  read_syslogref_conf_file()

  Syslog.open(PROJECTNAME, Syslog::LOG_PID | Syslog::LOG_NDELAY)
  
  #hide eye catch
  window.hide
  window.destroy

  #load some hidden window in background
  $treenode_win=TreeNodeWindow.new
  $event_win=EventWindow.new

  #by default first map is root map
  $map=nil

  map_window()

  #we have loaded all node
  $event_win.update_statusbar()
  $treenode_win.reload()

$threads=ThreadGroup.new()
$thread_list=Hash.new

if $config.port_mon
	#thread to listen to monitored port & heartbeat host
	add_monitoring_thread("Port monitoring",tmonitorport())
end
if $config.snmp_mon
	#thread to listen to monitored snmp oid refs
	add_monitoring_thread("Snmp monitoring",tmonitorsnmp())
end
if $config.custom_mon
	#thread to listen to monitored custom cmd
	add_monitoring_thread("Custom monitoring",tmonitorcustom())
end
if $config.broadcast_ping_state
	#thread to find new host in local network with broacast ping
	add_monitoring_thread("Local ping",tfindlocalhost())
	#thread to find new host in remote network with nmap ping
	add_monitoring_thread("Remote ping",t_find_remote_new_host())
end
if $config.mac_state
	#thread to find MAC address
	add_monitoring_thread("Find mac",tmacip())
end
if $config.node_resolving
	#thread to resolv ip into name
	add_monitoring_thread("Resolve ip to name",tresolvip())
	#thread to resolve ip into netbios name
	add_monitoring_thread("Resolve ip to netbios",tresolvnetbiosname())
	#thread to find os with fingerprinting
	add_monitoring_thread("Find os",tresolvos())
end

$syslog_capture=SyslogCapture.new()

end
