require "#{GNMSLIB}"+'module/ipcalc'
require 'net/http'

#front end module to use linux command
#module pour utiliser les outils Linux

#
#	download mac manufacturer list
#
def get_mac_manufacturer_list()
  if ! mac_manufacturer_list_exist?()
	ieee_addr="standards.ieee.org"
   ieee_url="/regauth/oui/oui.txt"

	local_file_path="#{CONF_DIR}"
	local_file_name="oui.txt"
	Net::HTTP.version_1_1

	tf=Tempfile.new(local_file_name)
	begin
 	  Net::HTTP.start(ieee_addr, 80) do |http|
   	response,=http.get(ieee_url)
		response.body.each do |str|
		str.chomp
		 if str.match('base\s16')
			  #line syntax is XXXXXX     (base 16)            name
			  str.scan(/(\w+)\s+\([^)]+\)\s+(.+)/) do |mac,manufacturer|
		       tf.write "#{mac}\t#{manufacturer}\n"
			  end
		  end
		 end	
    end
	rescue
		tf.close(true)
		return -1
	end
  end
	tf.close
  File.rename(tf.path, "#{local_file_path}/#{local_file_name}")
  return 0
end

#
#	return if mac manufacturer list exist
#
def mac_manufacturer_list_exist?()
	return FileTest.exist?("#{CONF_DIR}/oui.txt")
end

#
# Return nic manufacturer based on OUI
#
def find_manufacturer (mac)
	if FileTest.exist?("#{CONF_DIR}/oui.txt")

		if defined?($mac_manufacturer)
			if $mac_manufacturer[mac] != ""
				return $mac_manufacturer[mac]
			end
		else
			$mac_manufacturer=Hash.new
		end

		manufacturer_name="unknown"
		f=File.open("#{CONF_DIR}/oui.txt","r")
		begin
		while (line = f.readline)
        line.chomp
        if line.match("^#{mac}")
			result_t=line.split(/\s+/)
			manufacturer_name=result_t[1]
		  end
    	end
		rescue EOFError
    		f.close
		end
	   $mac_manufacturer[mac]=manufacturer_name
      return manufacturer_name
	else
		$log.warn("you need to download OUIs list, see pref")
		return nil
	end
end

#
#list port with nmap
#
def listport(ip)
	lp=`#{$config.nmap_path} -sU -sT #{ip} --host_timeout 60000 | grep open | grep "^[0-9]"`
	$_=lp
	gsub("/"," ")
	gsub("open","")
	gsub("|filtered","")
  	gsub("\ +"," ")
	return $_
end

#
#get nmap version
#
def nmap_version()
	if $config.nmap_path != ""
		regx=Regexp.new('Nmap version ([\d.]+) \(')
		ret=`#{$config.nmap_path} -V|grep Nmap`
		ret.scan(regx) do |val|
			return val[0]
		end
		return nil
	end
end

#
#	Return mac address of local machine
#
def maclocal(ipd)
  #we assume ipd exist

  netdev = `#{$config.ip_path} addr | grep "^[0-9]" | awk '{ print $2 }'`
  netdevt=netdev.split(/\n/)
  found = false
  nd=""

  for netd in netdevt
    nd=netd.chomp
    nd=nd[0,nd.size-1] #we delete : at the end
    ipnd = `#{$config.ip_path} addr show dev #{nd} | grep inet | awk '{ print $2 }'`
    ipndt = ipnd.split(/\n/)
    for ip in ipndt
      $_=ip
      gsub("/.*","")
      if ipd == $_
        found = true
        break #we have the right nd
      end
    end
    if found
      break
    end
  end
  ipdelamachine=`#{$config.ip_path} link show dev #{nd} | grep ether | awk '{ print $2 }'`
  iptab=ipdelamachine.chomp
  return iptab
end

#
# Return mac adress of the ip if in local arp table
#
def mac_tablelocal(ip)
#fixme
`ping -c 1 -W 1 #{ip}`
	lp=`arp -n #{ip} | grep #{ip} | awk '{print $3;}'`
	#there is no entry
	if lp.chomp == "--"
	  lp=""
	end
	return lp
end

#
#	Return ip adress of local machine
#
def iplocal
	ipdelamachine=`#{$config.ip_path} addr|grep "inet "| awk '{ print $2 }'`
	iptab=ipdelamachine.split(/\n/)
	return iptab
end

#
#	Return netmask adress of local machine
#
def masklocal()
	netmaskdelamachine=`#{$config.ip_path} addr|grep inet| awk '{ print $2 }'`
	netmaskdelamachine_t=netmaskdelamachine.split(/\n/)
	netmask_t=netmaskdelamachine_t[0].split(/\//)
	print "dans masklocal: ", netmask_t[1],"\n"
	return netmask_t[1] 
end

#
#ping broadcast
#
def broadcast_ping
	ipbroadcast=`#{$config.ping_path} 255.255.255.255 -b -c 2 -n 2>&1 | grep ^64 | awk '{ print $4 }'`
	$_=ipbroadcast
	gsub(":","")
	iptab=$_.split(/\n/)
	return iptab
	
	#iptab=ipbroadcast.split(/\n/)
	#for ip in iptab
	#  print ip[0,ip.size - 1],"\n"
end

#
# To know if an host is up, we use here ping perhaps in a near future nmap
#
def ping (ip)
	pip=`#{$config.ping_path} #{ip} -c 1 -n -W 4 | grep ^64`
	return pip!=""
end


#
#	verify by custom monitoring
#
def test_monitored_custom(ip)
	if $host[ip].custom_monitoring.size() > 0
		max_severity=UNKNOWN
		max_severity_description=nil
		$host[ip].custom_monitoring.each {|custom_monitoring|
			p custom_monitoring
			result_tmp=(eval custom_monitoring).new($host[ip])
			while (result_tmp.get_event_state_raw()==nil) do
				sleep(5)
			end			

			if $status_value[result_tmp.get_event_state_raw()] > max_severity
				max_severity=$status_value[result_tmp.get_event_state_raw()]
				max_severity_description=result_tmp.get_event_description()
			end
			if result_tmp.get_event_description() != nil
				$event_win.add_event("custom", get_level_from_status($status[result_tmp.get_event_state_raw()]), "#{ip}", "#{result_tmp.get_event_description()}")		
			end
		}
		if max_severity > $host[ip].custom_sev
			$host[ip].set_custom_severity(max_severity)
		end
	end
end

#
#	verify if oid ref threshold value is ok
#
def test_monitored_snmp(ip)
	if $host[ip].snmp_monitoring.size() > 0
		max_severity=UNKNOWN
		$host[ip].snmp_monitoring.each_value {|snmp_mon|
			snmp_oper="#{snmp_mon.oper()}"
			if "#{snmp_mon.oper()}" == "="
				snmp_oper="=="
			end
			begin
			expr_test="#{snmp_mon.value($host[ip])} #{snmp_oper} #{snmp_mon.threshold}"
p "snmp", max_severity, snmp_mon.raw_severity, expr_test
			if eval(expr_test)
				if $status_value[snmp_mon.raw_severity] > max_severity
					max_severity=$status_value[snmp_mon.raw_severity]
				end
		      $event_win.add_event("snmp", get_level_from_status($status[$status_value.index(max_severity)]), "#{ip}", "#{snmp_mon.oidref} matched")
			end
			rescue
				$log.error("Problem in SNMP request, validate #{snmp_mon.oidref} OID Reference")
				return
			end
		}
		#set the snmp severity to max find for this host
puts "final", max_severity, $host[ip].snmp_sev
		if max_severity > $host[ip].snmp_sev
			$host[ip].set_snmp_severity(max_severity)
		end
	end
end
#
# verify if monitored ports are always open
# return true if ok
def test_monitored_ports(ip)
#we assume here $host[ip].port is not nil
	if $host[ip].service.size() == 0
		return true
	end

	puts "monitored ports for #{ip}"
	ip_port=""
	$host[ip].service.each_value{|serv|
	  if serv.monitor.to_i() == 1
		 proto="U"
		 if serv.protocol == "tcp"
			 proto="T"
		 end
		 ip_port+="#{proto}:#{serv.port},"
	  end
	}
	ip_port.chop!()
	if ip_port == ""
		#we dont monitor ports for this node
		return true
	end
   ports = ip_port.split(/,/).dup
	lp=`#{$config.nmap_path} -sT -sU #{ip} --host_timeout 60000 -p #{ip_port} 2>/dev/null | grep "^[0-9]"`

	llp=lp.split(/\n/)
    for pt in llp
		ptlign=pt.split(/\s+/)
		portprotocol=ptlign[0]
		portstate=ptlign[1]
		if portstate.match("^open")
      	ptt = portprotocol.split(/\//)
      	if ptt[1] == "tcp"
      	  pt="T:#{ptt[0]}"
      	else
      	  pt="U:#{ptt[0]}"
      	end
      	i=0
      	for ptlo in ports
      	  if ptlo == pt
         	 ports.delete ptlo
         	 break
      	  end
      	  i+=1
      	end
		end
    end
    if ports.size != 0 #we have port which dont respond
      str=ports.join(" ")
		#nmap protocol/port syntax to human easy readable!
		str.gsub!(/T:/,"tcp/")
		str.gsub!(/U:/,"udp/")
      $event_win.add_event("port", "ALERT", "#{ip}", "#{str} not responding")
	end
	return ports.size == 0
end

#
# Return all host seems to be up (responding to icmp) from a given network (class c)
# netmork submitted as 0.0.0.0/24
def broadcast_remote_ping(network, mask)
	puts ">>>>>>>>in broadcast_remote_ping #{network} #{mask}"
  result=[]
  if $config.nmap_path == ""
    return result
  end
	if $config.nmap_vers.to_f() >= 4
	  res=`#{$config.nmap_path} --send-ip -PE -sP -n "#{network}/#{mask}" | grep "^H.*$" | awk '{print $2}'`
	else
	  res=`#{$config.nmap_path} -PE -sP -n "#{network}/#{mask}" | grep "^H.*$" | awk '{print $2}'`
	end
  lres=res.split(/\n/)
  for ip in lres
    result.push ip
  end
  return result
end

#
# Return all host seems to be up (responding to icmp) from a given range IP addr
def range_remote_ping(ip_begin, ip_end)
  result=[]
  if $config.nmap_path == ""
    return result
  end
  res=`#{$config.nmap_path} -sP -n "#{ip_begin}-#{ip_end}" | grep "^H.*$" | awk '{print $2}'`
  lres=res.split(/\n/)
  for ip in lres
    result.push ip
  end
  return result
end

#
# OS Fingerprint
#
def osfingerprint(ip)

  print "dans xprobe2 ",ip,"\n"

  if $config.xprobe_path != ""
    #with xprobe version 1
    #res=`#{$config.xprobe_path} #{ip} 2>&1 | grep "FINAL"`
    #with xprobe version 2
    res=`#{$config.xprobe_path} -c /etc/xprobe2/xprobe2.conf -m 1 #{ip} 2>&1 | grep "Running OS"`
    print "xprobe2>>>>",res,"\n"

if (res == "")
	res=`#{$config.nmap_path} -O #{ip} --host_timeout 10000 2>&1 | grep "Running: "`
	print "nmap>>>",res,"\n"
end

    if (res!="")
    if (res.index("Linux")!=nil)
  	  res="linux"
      return res
    end
  	if (res.index("Win")!=nil)
  	  res="windows"
      return res
	end
    if (res.index("HPUX")!=nil)
  	  res="hp"
      return res
    end
    if (res.index("3Com")!=nil)
      res="3com"
      $host[ip].type="switch"
      return res
    end
    if (res.index("Cisco")!=nil)
      res="cisco"
      $host[ip].type="router"
      return res
    end
    if (res.index("BSD")!=nil)
      res="freebsd"
      return res
    end    
    end
  end
  return ""
end

#
# Return the service name associate to the port give in argument
# first is protocol, second is port
def service_name(protocol,port)
  sn = `cat /etc/services | awk '{print $2, $1;}' | grep "^#{port}/#{protocol}" | awk '{print $2;}'`
  return sn.chomp
end

#
# find netbios name
#
def ip2netbiosname(ip)
  if $config.nmblookup_path == ""
    return nil
  end
  res=`#{$config.nmblookup_path} -A "#{ip}" | grep ACTIVE | awk '{print $1;}'`
  rest=res.split(/\n/)
  return rest[0] 
end

#
# convert bitmask address to netmask address
#
def bit2netmask(int)
  if int==0
    return "0.0.0.0"
  end
  return IPCalc.bits_to_netmask(int)
end

#
# convert netmask address to bitmask address
#
def net2bitmask(netmask)
  if netmask =="0.0.0.0"
    return 0
  end
  return IPCalc.netmask_to_bits(netmask)
end

#
#  return network address and compress mask class
#
def getnetwork_and_mask(ip)
  if ip == "" || ip == nil
    return nil
  end
  return IPCalc.get_network_and_mask(ip)
end
