require "#{GNMSLIB}"+'module/gui/color_macro'
require "#{GNMSLIB}"+'module/service'

class Node

NONAME_PROPERTY="(noname)"
NOARP_PROPERTY="(noarp)"
NODESCR_PROPERTY="(nodesc)"

   attr_accessor :ip, :mask, :service, :custom_monitoring, :snmp_monitoring, :syslog_monitoring, :status, :x, :y, :icon, :description, :snmpread, :snmpwrite, :type, :os, :node_view, :map, :custom_monitor
	attr_reader :avail_sev, :snmp_sev, :custom_sev, :trap_sev

   def initialize (ip="",netmask=24, mp=nil, serv=nil, s=UNKNOWN, xd=0 ,yd=0, i="", d=NODESCR_PROPERTY, sr="public", sw="public", t=nil, o="unknown")

      @ip=ip
      @mask=netmask
		if serv
			#serv must be a hash of service class
			@service=serv.dup
		else
	      @service=Hash.new
		end

		@snmp_monitoring=Hash.new
		@custom_monitoring=[]
		@syslog_monitoring=Hash.new

      @status=s
		@avail_sev=UNKNOWN
		@snmp_sev=UNMANAGED
		@custom_sev=UNMANAGED
		@trap_sev=UNMANAGED

      @x=xd
      @y=yd
      @nb_interface=1
		@links=[]		
      @type=t

      if i == "" || i == nil
			if $config.host_icon_view=="os"
		  		@icon = "os/unknown.png"
			else
				#type
				if (@type != nil) && (@type != "")
					@icon = "#{find_extension("type/#{@type}")}"
				end
			end
      else
  	  		@icon=i
      end

      if d == "" || d == nil
    		@description=""
      else
      	@description=d
      end
      @snmpread=sr
      @snmpwrite=sw
      @os=o
      #ip and map could not be the same
      if mp != ip
	if mp == nil && ip != ROOTMAPADDR
		#we want the node on the root network
		@map=ROOTMAPADDR
	else
		if ip == ROOTMAPADDR
			@map=nil
		else
			@map=mp
		end
	end
      else
			@map=ROOTMAPADDR
      end

	if $treenode_win.visible?
		$treenode_win.add_row(self)
	end
		
  end

	def del_all_links()
		get_links().each {|lnk|
			lnk.node1.del_link(lnk)
			lnk.node2.del_link(lnk)
			$links.delete(lnk)
			lnk.remove_link()
		}
	end

	def get_links()
		return @links
	end

	def nb_links()
		return @links.size
	end

	def add_link(lnk)
		if @links.index(lnk) == nil
			@links.push(lnk)
		end
	end

	def del_link(lnk)
		@links.delete(lnk)
	end

	#
	#	update status based on the avail, snmp, custom severities
	#
	def refresh_status()
	#it s $status_value
		#print @avail_sev," ", @snmp_sev, " ",@custom_sev,"\n"
		if (@avail_sev == CRITICAL) ||  (@snmp_sev == CRITICAL) || (@custom_sev == CRITICAL) || (@trap_sev == CRITICAL)
			change_status(CRITICAL)	
		else
		  cpt=0
		  if @avail_sev > UNMANAGED
			  cpt+=1
		  end
		  if @snmp_sev > UNMANAGED
			  cpt+=1
		  end
		  if @custom_sev > UNMANAGED
			  cpt+=1
		  end
		  if @trap_sev > UNMANAGED
			  cpt+=1
		  end

		  if cpt>0
			  val=(@avail_sev+@snmp_sev+@custom_sev+@trap_sev)/cpt
  			  #print @avail_sev," ", @snmp_sev, " ",@custom_sev,"\n"
			  #print "cpt: ",cpt, " val ",val.floor(),"\n"
			  val_result=val.floor()
			  val_result*=-1 if val_result<0
			  change_status(val_result)
			else
				if @avail_sev+@snmp_sev+@custom_sev+@trap_sev < 0
					change_status(UNKNOWN)
				else
					change_status(UNMANAGED)
				end
		  end
		end
	end

	def hide_label()
		@node_view.hide_text()
	end

	def show_label()
		@node_view.show_text()
	end

	def set_label_visibility()
		if to_b($config.show_label) == false
			hide_label()
			return false
		else
			show_label()
			return true
		end	
	end

	def set_avail_severity(sev)
		@avail_sev=sev
		refresh_status()
	end

	#sev is an index $status_value
	def set_snmp_severity(sev)
		@snmp_sev=$status_value[sev]
		refresh_status()
	end

	def set_custom_severity(sev)
		@custom_sev=sev
		refresh_status()
	end

	def set_trap_severity(sev)
puts "set trap severrity",sev
		@trap_sev=$status_value[sev]
		refresh_status()
	end
	
	def set_all_severity(sev)
		@avail_sev=sev
		@snmp_sev=sev
		@custom_sev=sev
		refresh_status()
	end

  def service_key(protocol, port)
	  return "#{protocol}/#{port}"
  end

  def add_service(protocol, port, name="?", monitor_flag=1)
	  if @service[service_key(protocol, port)] == nil
		  @service[service_key(protocol, port)]=Service.new(protocol,port,name,monitor_flag)
	  end
  end

  def del_service(protocol, port)
	  if @service[service_key(protocol, port)] != nil
		  @service[service_key(protocol, port)].delete()
	  end
  end

  def add_snmp_monitoring_init(oidref_name, op, val, sev)
	  if @snmp_monitoring[oidref_name] == nil
		  @snmp_monitoring[oidref_name]=SnmpMonitoring.new(oidref_name, op, val, sev)
		  $oid_refs[oidref_name].add_use()
	  end
  end

  def add_snmp_monitoring(oidref_name, op, val, sev)
	  if @snmp_monitoring[oidref_name] == nil
		  @snmp_monitoring[oidref_name]=SnmpMonitoring.new(oidref_name, SnmpMonitoring::TESTOP.index(op), val, $status.index(sev))
		  $oid_refs[oidref_name].add_use()
	  end
  end

  def del_snmp_monitoring(oidref_name)
	  if @snmp_monitoring[oidref_name] != nil
		  $oid_refs[oidref_name].del_use()
		  @snmp_monitoring[oidref_name].delete()
	  end
  end

  def add_custom_monitoring(custom_mon_name)
	  if ($custom_monitor.index(custom_mon_name) != nil)&&(!@custom_monitoring.include?(custom_mon_name))
		  @custom_monitoring.push(custom_mon_name)
	  end
  end

  def del_custom_monitoring(custom_mon_name)
	  if @custom_monitoring.include?(custom_mon_name)
		  @custom_monitoring.delete(custom_mon_name)
	  end
  end

  def add_syslog_monitoring(syslog_ref_name, sev)
	  if @syslog_monitoring[syslog_ref_name] == nil
puts "add_syslog_monitoring ", sev, $status.index(sev)
		  @syslog_monitoring[syslog_ref_name]=SyslogMonitoring.new(syslog_ref_name, $status.index(sev))
		  $syslog_refs[syslog_ref_name].add_use()
	  end
  end

  def del_syslog_monitoring(syslog_ref_name)
	  if @syslog_monitoring[syslog_ref_name] != nil
		  $syslog_refs[syslog_ref_name].del_use()
		  @syslog_monitoring[syslog_ref_name].delete()
	  end
  end

	#
	#	set map for changing map of a node
	#
	def set_map(map)
		if map != @map
			$network[@map].del_host(ip)
			@map=map
			$network[@map].add_host(self)
			@node_view.reparent($network[@map].map_view)
		end
	end

	#
	#	clear all services
	#
   def clear_service()
		@service=Hash.new
		#@avail_sev=UNMANAGED
   end

	#
	#	clear all snmp monitoring
	#
   def clear_snmp_monitoring()
		@snmp_monitoring=Hash.new
		#@snmp_sev=UNMANAGED	
   end

	#
	#	clear all custom monitoring
	#
   def clear_custom_monitoring()
		@custom_monitoring=[]
		#@custom_sev=UNMANAGED
   end

	#
	#	clear all trap monitoring
	#
   def clear_trap_monitoring()
		@syslog_monitoring=Hash.new
		#@trap_sev=UNMANAGED	
   end

  def description?()
	   return description != "(nodesc)"
   end
 
#
#	return a point corresponding to the position of the node
#
def point_pos()
	return [@x, @y]
end


end #class
